
#define TWEN 2
#define TWIE 0
#define TWINT 7
#define TWEA 6
#define TWSTA 5
#define TWSTO 4
#define TWWC 3


#include <delay.h>
#include <mega16.h>    
#include <math.h>    
#include <bcd.h>
typedef unsigned char byte;

char rr1,rr2,rr3,rr4,rr5,temp,mh,ml,sh,sl,hh,hl;
long ps;
flash byte char0[]={
0xC0,
0xF9,
0xA4,
0xB0,
0x99,
0x92,
0x82,
0xf8,
0x80,
0x90,                   // '9'
0xBF,                   // '-'
0xFF,                   //  
0b10011100,             //  
0b11000110,              // 'C'
0b10010010,              // 'S'
0b10000110,              // 'E'
0b10101111,              // 'r'
0b11000010              // 'G'

}    ;

void twi_start(void) {
TWCR = (1<<TWEA)|(1<<TWINT)|(1<<TWSTA)|(1<<TWEN);

while (!(TWCR & (1<<TWINT))) {; }
}

void twi_stop(void) {
TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
}

void twi_write(unsigned char _data)
{
TWDR = _data;
TWCR = (1<<TWINT)|(1<<TWEN);

while (!(TWCR & (1<<TWINT))) {;}
}

unsigned char twi_read(unsigned char _ack) {
unsigned char _data;

if (_ack==1)
{
TWCR = (1<<TWEA)|(1<<TWINT) | (1<<TWEN);
}
else
{
TWCR = (1<<TWINT) | (1<<TWEN);
}
while (!(TWCR & (1<<TWINT)))
{
}
_data = TWDR;
return _data;
}

void rtc_init(unsigned char rs,unsigned char sqwe,unsigned char out) {
rs&=3;
if (sqwe) rs|=0x10;
if (out) rs|=0x80;
twi_start();
twi_write(0xd0);
twi_write(7);
twi_write(rs);
twi_stop();
}

void rtc_set_time(unsigned char hour,unsigned char min,unsigned char sec) {
twi_start();
twi_write(0xd0);
twi_write(0);
twi_write(sec);
twi_write(min);
twi_write(hour);
twi_stop();
}

void rtc_set_date(unsigned char date,unsigned char month,unsigned char year) {
twi_start();
twi_write(0xd0);
twi_write(4);
twi_write(date);
twi_write(month);
twi_write(year);
twi_stop();
}

void rtc_get_time(unsigned char *hour,unsigned char *min,unsigned char *sec) {
twi_start();
twi_write(0xd0);
twi_write(0);
twi_start();
twi_write(0xd1);
*sec=twi_read(1);
*min=twi_read(1);
*hour=twi_read(0);
twi_stop();
}

void rtc_get_date(unsigned char *date,unsigned char *month,unsigned char *year) {
twi_start();
twi_write(0xd0);
twi_write(4);
twi_start();
twi_write(0xd1);
*date=twi_read(1);
*month=twi_read(1);
*year=twi_read(0);
twi_stop();
}

unsigned char rtc_read(unsigned char address) {
unsigned char data;
twi_start();
twi_write(0xd0);
twi_write(address);
twi_start();
twi_write(0xd1);
data=twi_read(0);
twi_stop();
return data;
}

void rtc_write(unsigned char address, unsigned char data) {
twi_start();
twi_write(0xd0);
twi_write(address);
twi_write(data);
twi_stop();
}

unsigned char h,m,s;

// External Interrupt 0 service routine
interrupt [EXT_INT0] void ext_int0_isr(void)
{
// Place your code here
rtc_get_time(&h,&m,&s);

//hh=bcd2bin(h)/10;
//hl=bcd2bin(h)%10;
mh=bcd2bin(m)/10;
ml=bcd2bin(m)%10;
sh=bcd2bin(s)/10;
sl=bcd2bin(s)%10;

}



void display (void)
{
    rr1=mh;
    rr2=ml;
    rr3=10;
    rr4=sh;
    rr5=sl;
    ps=240;


        PORTB=0b11110000;
        PORTA=char0[rr5];
        for (temp=0;temp<ps;temp++) {delay_us(8);};
        PORTB=0b11100000;
        delay_us(50);

        PORTB=0b11101000;
        PORTA=char0[rr4];
        for (temp=0;temp<ps;temp++) {delay_us(8);};
        PORTB=0b11100000;
        delay_us(50);

        PORTB=0b11100100;
        PORTA=char0[rr3];
        for (temp=0;temp<ps;temp++) {delay_us(8);};
        PORTB=0b11100000;
        delay_us(50);

        PORTB=0b11100010;
        PORTA=char0[rr2];
        for (temp=0;temp<ps;temp++) {delay_us(8);};
        PORTB=0b11100000;
        delay_us(50);

        PORTB=0b11100001;
        PORTA=char0[rr1];
        for (temp=0;temp<ps;temp++) {delay_us(8);};
        PORTB=0b11100000;
        delay_us(50);

        PORTB=0b11100000;  //  
} 


//    0
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
//display(); 
}



//************* ****************************

void main(void)
{
char i;

DDRB=0b10011111;   // 
PORTB=0b00000000;        // B   
DDRB=0b11111111;   // 
PORTB=0b00000000;        // B   

PORTA=0xA4;  // A  
DDRA=0xFF;   //  

PORTC=0x00;
DDRC=0x00;

PORTD=0b10000000;
DDRD=0b00000000;

// Timer/Counter 0 initialization
TCCR0=0x04;
TCNT0=0x00;
OCR0=0x00;
//******** / 1************
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Rising Edge
// INT1: Off
// INT2: Off
GICR|=0x40;
MCUCR=0x03;
MCUCSR=0x00;
GIFR=0x40;


//*********** / 1 ()****
TIMSK=0x01;  //   1,2
  
// 2 Wire Bus initialization
// Generate Acknowledge Pulse: Off
// 2 Wire Bus Slave Address: 0h
// General Call Recognition: Off
// Bit Rate: 400,000 kHz
TWSR=0x00;
TWBR=0x0C;
TWAR=0x00;
TWCR=0x04;


#asm("sei")  //   

while (1)
        {                                                 
PORTB.7=1;
if (PIND.7==0)
             {
        PORTB.7=0;
        rtc_set_date(0x31,0x12,0x10);
        while (i<5) {i++;}
        rtc_set_time(0x15,0x08,0x00);
        while (i<5) {i++;}
        rtc_init(0,1,0);
             }

        
        };


}
